{% extends "admin/change_list.html" %}
{% load i18n %}

{% if not is_popup %}
    {% block breadcrumbs %}
        <div class="breadcrumbs">
            <a href="../../">
                {% trans "Home" %}
            </a>
            &rsaquo; 
            <a href="/admin/{{ related_opts.app_label }}/">
                {{ related_opts.app_label|capfirst }}
            </a>
            &rsaquo; 
            {{ cl.opts.verbose_name_plural|capfirst }}
        </div>
    {% endblock %}
{% endif %}
        
{% block content_title %}<h1>Arrange {{ related_opts.verbose_name_plural }}</h1>{% endblock %}
    
{% block object-tools %}
    {% if has_add_permission %}
        <ul class="object-tools">
            <li><a href="{{ list_url }}" class="historylink">List</a></li>
            <li>
                <a href="{{ add_url }}" class="addlink">
                    {% blocktrans with related_opts.verbose_name as name %}Add {{ name }}{% endblocktrans %}
                </a>
            </li>
        </ul>
    {% endif %}
{% endblock %}
